import argparse
import giftsnippet


def main():
    parser = argparse.ArgumentParser(description="Replace gift-snippets in your gift file with highlighted code images")
    parser.add_argument('file', help='Provide a gift file to replace gift-snippets with highlighted code images')
    args = parser.parse_args()
    giftsnippet.process_gift_file(args.file)





from .highlight import get_data_uri_snippet
import re
import fileinput
import os


def process_gift_file(file):
    base_dir = get_base_dir(file)
    lines = find_snippet_lines(file)

    replace_line_indices = []
    new_lines = []

    for line in lines:
        data_uri = get_data_uri_snippet(base_dir + "/" + line['snippet_file'])
        replace_line_indices.append(line['line_index'])
        new_lines.append('{}<img data-gift-snippet\\="{}" src\\="{}">\n'.format(line['whitespace'], line['snippet_file'], data_uri))

    replace_lines(file, replace_line_indices, new_lines)


def find_snippet_lines(file):
    assert os.path.exists(file)
    lines = []

    with open(file) as file_object:
        for line_index, line in enumerate(file_object):
            m = re.match('^( *)<img data-gift-snippet\\\="([^"]*?)"', line)

            if m:
                lines.append({
                    "line": line,
                    "line_index": line_index,
                    "whitespace": m.group(1),
                    "snippet_file": m.group(2)
                })

    return lines


def get_base_dir(file):
    return os.path.dirname(file)


def replace_lines(file, replace_line_indices=[], new_lines=[]):
    assert len(replace_line_indices) >= 1 and len(replace_line_indices) == len(new_lines)

    for line_index, line in enumerate(fileinput.input(file, inplace=True)):
        if line_index in replace_line_indices:
            index = replace_line_indices.index(line_index)
            print(new_lines[index], end='')
        else:
            print(line, end='')




import pathlib
import pkg_resources
import setuptools

with pathlib.Path('./requirements.txt').open() as requirements_txt:
    install_requires = [
        str(requirement)
        for requirement
        in pkg_resources.parse_requirements(requirements_txt)
    ]

def readme():
    with open('README.md') as f:
        return f.read()

setuptools.setup(name='giftsnippet',
      version='0.1.1',
      description='Integrate highlighted source code images with data uris into your gift questions',
      long_description=readme(),
      long_description_content_type='text/markdown',
      keywords='gift moodle code-formatting',
      url='http://github.com/stefanhuber/giftsnippet',
      author='Stefan Huber',
      author_email='mail@stefanhuber.at',
      license='BSD',
      packages=['giftsnippet'],
      install_requires=install_requires,
      test_suite="giftsnippet.tests",
      include_package_data=True,
      zip_safe=False,
     entry_points={
         'console_scripts': ['giftsnippet=giftsnippet.console:main'],
     })



import base64
import os
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import get_formatter_by_name


def get_data_uri_snippet(file, lexer='python', gift_escape=True):
    snippet = load_snippet(file)
    image = highlight(snippet, get_lexer_by_name(lexer), get_formatter_by_name('png'))
    data_uri = 'data:image/png;base64,' + base64.b64encode(image).decode("utf-8")

    # : and = must be escaped inside the question content for gift and moodle import
    if gift_escape:
        data_uri = data_uri.replace(":", "\\:")
        data_uri = data_uri.replace("=", "\\=")

    return data_uri


def load_snippet(file):
    assert os.path.exists(file)
    with open(file) as file_object:
        return file_object.read()
